#include "xapp.h"
#include "xgraph.h"
#include "csystem.h"

#ifdef __cplusplus
extern "C" {
#endif
  #include <math.h>
  #include <stdio.h>
#ifdef __cplusplus
}
#endif

typedef struct {
          Window window;
	  csystem *item;
	} cwindowlist;

#define MAXWINDOWS 1024;

static cwindowlist *windows;
static unsigned int nwindows;
static cwindowlist topwindow;
static Widget topshell;
static Widget wx,wy,wz,wc;
static XtAppContext ctx;
static Display *dpy;
static unsigned short int LUT[256*3];
static int ILUT[256];

csystem::csystem() {
  data=NULL;
  ndata=0;
  dim=0;
  xaxis=-1;
  yaxis=-1;
  zaxis=-1;
  caxis=-1;
  axiss=NULL;
  units=NULL;
}

void csystem::SetData(double **src,unsigned long srcn,unsigned int srcdim,int clear) {
  unsigned long i;
  
  if ((data!=NULL) && (clear)) {
    for (i=0;i<ndata;i++)
      delete data[i];
    delete data;
  }

  ndata=srcn;
  dim=srcdim;
  data=src;
}

unsigned int csystem::X() {
  return xaxis;
}

unsigned int csystem::Y() {
  return yaxis;
}

unsigned int csystem::Z() {
  return zaxis;
}

unsigned int csystem::C() {
  return caxis;
}

int csystem::Type() {
  return type;
}

void csystem::X(unsigned int i) {
  xaxis=i;
}

void csystem::Y(unsigned int i) {
  yaxis=i;
}

void csystem::Z(unsigned int i) {
  zaxis=i;
}

void csystem::C(unsigned int i) {
  caxis=i;
}

char *csystem::axis(int i) {
  return axiss[i];
}

char *csystem::unit(int i) {
  return units[i];
}

double csystem::Data(unsigned long z,unsigned int s) {
  return data[z][s];
}

unsigned long csystem::NData() {
  return ndata;
}

unsigned int csystem::Dim() {
  return dim;
}

csystem::~csystem() {
  unsigned long i;
  
  if (data!=NULL) {
    for (i=0;i<ndata;i++)
      delete data[i];
    delete data;
  }
}

void csystem::CopyStrings(csystem *src) {
  int i;

  if (units!=NULL) {
    for (i=0;i<dim;i++) {
      delete units[i];
      delete axiss[i];
    }
    delete axiss;
    delete units;
  }

  units=new char* [src->dim];
  axiss=new char* [src->dim];
  for (i=0;i<src->dim;i++) {
    units[i]=new char [strlen(src->units[i])+1];
    axiss[i]=new char [strlen(src->axiss[i])+1];
    strcpy(units[i],src->units[i]);
    strcpy(axiss[i],src->axiss[i]);
  }
}

void csystem::LoadFormatted(char *fname,int clear) {
  unsigned long i,j;
  FILE *f;
  char c;

  if ((data!=NULL) && (clear)) {
    for (i=0;i<ndata;i++)
      delete data[i];
    delete data;
  }
  if ((units!=NULL) && (clear)) {
    for (i=0;i<dim;i++) {
      delete units[i];
      delete axiss[i];
    }
    delete axiss;
    delete units;
  }
  
  if ((f=fopen(fname,"r"))!=NULL) {
    // number of data and columns
    fscanf(f,"%d %d",&ndata,&dim);

    // the names and units
    units=new char* [dim];
    axiss=new char* [dim];
    for (i=0;i<dim;i++) {
      units[i]=new char [8];
      axiss[i]=new char [8];
      fscanf(f,"%s %s",axiss[i],units[i]);
    }
    
    // the data
    data=new double* [ndata];
    for (i=0;i<ndata;i++) {
      data[i]=new double [dim];
      for (j=0;j<dim;j++) {
	fscanf(f,"%lf",&data[i][j]);
      }
    }

    fclose(f);
  } else
    printf("Error opening %s.\n",fname);
}

cwindowlist FindItem(csystem *which) {
  unsigned int i;
  
  for (i=0;i<nwindows;i++) 
    if (windows[i].item==which) return windows[i];
  
  return topwindow;
}

cwindowlist FindItem(Window w) {
  unsigned int i;

  for (i=0;i<nwindows;i++) {
    if (windows[i].window==w) return windows[i];
  }
  return topwindow;
}

void drawpoint(int x2d,int y2d,int c2d,int ptype) {
  char s[256];

  setcolor(c2d);
  if (ptype & CDOT)
    putpixel(x2d,y2d,c2d);
  if (ptype & CCIRC)
    fillcircle(x2d,y2d,2);
  if (ptype & CCROSS) {
    line(x2d-4,y2d,x2d+4,y2d);
    line(x2d,y2d-4,x2d,y2d+4);
  }
  if (ptype & CQUAD)
    bar(x2d-2,y2d-2,x2d+2,y2d+2);
}

inline double sqr(double x) { return x*x; }

void cdraw(Window w,int reason) {
  double minx=0,maxx=0,miny=0,maxy=0,minz=0,maxz=0,minc=0,maxc=0,mind=0;
  unsigned long i,j;
  int k,l;
  int xi,yi,zi,ci,ii;
  int x2d,y2d,z2d,c2d;
  double x,y,z,c,d;
  double scalex,scaley,scalez,scalec;
  double deltax,deltay,deltaz,deltac;
  csystem *csys;
  char s[256];

  topwindow=FindItem(w);
  csys=topwindow.item;

  xi=csys->X();
  yi=csys->Y();
  zi=csys->Z();
  ci=csys->C();
  
  if (xi>=0) minx=maxx=csys->Data(0,xi);
  if (yi>=0) miny=maxy=csys->Data(0,yi);
  if (zi>=0) minz=maxz=csys->Data(0,zi);
  if (ci>=0) minc=maxc=csys->Data(0,ci);
    
  for (i=1;i<csys->NData();i++) {
    if (xi>=0) {
      if (csys->Data(i,xi)<minx) minx=csys->Data(i,xi);
      if (csys->Data(i,xi)>maxx) maxx=csys->Data(i,xi);
    }
    if (yi>=0) {
      if (csys->Data(i,yi)<miny) miny=csys->Data(i,yi);
      if (csys->Data(i,yi)>maxy) maxy=csys->Data(i,yi);
    }
    if (zi>=0) {
      if (csys->Data(i,zi)<minz) minz=csys->Data(i,zi);
      if (csys->Data(i,zi)>maxz) maxz=csys->Data(i,zi);
    }
    if (ci>=0) {
      if (csys->Data(i,ci)<minc) minc=csys->Data(i,ci);
      if (csys->Data(i,ci)>maxc) maxc=csys->Data(i,ci);
    }
  }
  deltax=maxx-minx;
  deltay=maxy-miny;
  deltaz=maxz-minz;
  deltac=maxc-minc;
  if ((reason==ResizeRequest) || (reason==Expose)) {
    setwindow(w);
    if (fabs(deltac)>1e-10) 
      scalec=(double)getmaxcolor()/deltac;
    else
      deltac=1.0;
    setcolor(xWHITE);
    rectangle(80,16,getmaxx()-106,getmaxy()-32);

    switch (csys->Type() & 3) {
      case C1D: scaley=1.0;
	        if (fabs(deltay)>1e-10) 
		  scaley=(double)(getmaxy()-48)/deltay;
                sprintf(s,"%s %s",csys->axis(csys->Y()),csys->unit(csys->Y()));
                outtextxy(10,16+(getmaxy()-48)/2,s);
                if ((miny<=0) && (maxy>=0))  
		  line(80,getmaxy()-32-(int)(scaley*(-miny)),
		       getmaxx()-106,getmaxy()-32-(int)(scaley*(-miny)));
                for (i=0;i<csys->NData();i++) {
		  x2d=(int)((double)(getmaxx()-186)*(double)i/(double)(csys->NData()))+80;
		  y2d=getmaxy()-32-(int)(scaley*(csys->Data(i,yi)-miny));
		  if (csys->Type() & CCOL)
		    c2d=ILUT[(int)(scalec*(csys->Data(i,ci)-minc))];
		  else
		    c2d=xWHITE;
		  drawpoint(x2d,y2d,c2d,csys->Type());
		}
	        setcolor(xWHITE);
	        sprintf(s,"%+lf",miny);
                outtextxy(0,getmaxy()-32+8,s);
                sprintf(s,"%+lf",maxy);
                outtextxy(0,16+8,s);
                break;
      case C2D: scalex=1.0;
                scaley=1.0;
                sprintf(s,"%s %s",csys->axis(csys->X()),csys->unit(csys->X()));
                outtextxy(80+(getmaxx()-186)/2,getmaxy()-5,s);
                sprintf(s,"%s %s",csys->axis(csys->Y()),csys->unit(csys->Y()));
                outtextxy(10,16+(getmaxy()-48)/2,s);
                if (fabs(deltax)>1e-10) 
		  scalex=(double)(getmaxx()-186)/deltax;
                if (fabs(deltay)>1e-10) 
		  scaley=(double)(getmaxy()-48)/deltay;
                if ((maxx>=0) && (minx<=0))
		  line(80+(int)(scalex*(-minx)),16,
		       80+(int)(scalex*(-minx)),getmaxy()-32);
                if ((maxy>=0) && (miny<=0))
		  line(80,getmaxy()-32-(int)(scaley*(-miny)),
		       getmaxx()-106,getmaxy()-32-(int)(scaley*(-miny)));
                for (i=0;i<csys->NData();i++) {
		  x2d=80+(int)(scalex*(csys->Data(i,xi)-minx));
		  y2d=getmaxy()-32-(int)(scaley*(csys->Data(i,yi)-miny));
		  if (csys->Type() & CCOL)
		    c2d=ILUT[(int)(scalec*(csys->Data(i,ci)-minc))];
		  else
		    c2d=xWHITE;
		  drawpoint(x2d,y2d,c2d,csys->Type());
		}
                setcolor(xWHITE);
                sprintf(s,"%+lf",minx);
                outtextxy(80,getmaxy()-5,s);
                sprintf(s,"%+lf",maxx);
                outtextxy(getmaxx()-126,getmaxy()-5,s);
                sprintf(s,"%+lf",miny);
                outtextxy(0,getmaxy()-32+8,s);
                sprintf(s,"%+lf",maxy);
                outtextxy(0,16+8,s);
                break;
      case C3D:
                break;
    }
    if (csys->Type() & CCOL) {
      setcolor(xWHITE);
      sprintf(s,"%s %s",csys->axis(csys->C()),csys->unit(csys->C()));
      outtextxy(getmaxx()-90,10,s);
      setcolor(xWHITE);
      for (i=16;i<getmaxy()-32;i++) {
	k=(int)((double)(i-16)/(double)(getmaxy()-48)*256.0);
	setcolor(ILUT[k]);
	line(getmaxx()-90,getmaxy()-i-16,getmaxx()-80,getmaxy()-i-16);
      }
      setcolor(xWHITE);
      rectangle(getmaxx()-90,16,getmaxx()-80,getmaxy()-32);
      sprintf(s,"%lf",maxc);
      outtextxy(getmaxx()-78,32,s);
      sprintf(s,"%lf",minc);
      outtextxy(getmaxx()-78,getmaxy()-32,s);
    }
  } else if (reason=ButtonPress) {
    setwindow(w);
    setcolor(xBLACK);
    bar(80,0,getmaxx()-106,15);
    getpointer(&k,&l);
    x=(double)k-80.0;
    y=(double)l-16.0;
    if ((x>=0) && (x<=getmaxx()-186) && (y>=0) && (y<=getmaxy()-48)) {
      if ((csys->Type() & 3)==1) 
	x=x/(double)(getmaxx()-186)*deltax+minx;
      else
	x=x/(double)(getmaxx()-186);
      y=maxy-y/(double)(getmaxy()-48)*deltay;
      for (i=0;i<csys->NData();i++) {
	switch (csys->Type() & 3) {
  	  case C1D: d=sqrt(sqr((double)i/(double)csys->NData()-x)+sqr(y-csys->Data(i,yi))); break;
	  case C2D: d=sqrt(sqr(x-csys->Data(i,xi))+sqr(y-csys->Data(i,yi))); break;
	}
	if (i==0) {
	  mind=d;
	  j=0;
	} else if (d<mind) {
	  mind=d;
	  j=i;
	}
      }
      if (csys->Type() & CCOL) {
	switch (csys->Type() & 3) {
	  case C1D: sprintf(s,"%d: (%+lf,%+lf)",j,csys->Data(j,yi),csys->Data(j,ci)); break;
	  case C2D: sprintf(s,"%d: (%+lf,%+lf,%lf)",j,
			    csys->Data(j,xi),csys->Data(j,yi),csys->Data(j,ci));
	            break;
	}
      } else
	switch (csys->Type() & 3) {
	  case C1D: sprintf(s,"%d: (%+lf)",j,csys->Data(j,yi)); break;
	  case C2D: sprintf(s,"%d: (%+lf,%+lf)",j,csys->Data(j,xi),csys->Data(j,yi));
	            break;
	}
      setcolor(xWHITE);
      outtextxy(90,10,s);
      if (fabs(deltay)>1e-10) 
	scaley=(double)(getmaxy()-48)/deltay;
      else
	scaley=1.0;
      switch (csys->Type() & 3) {
        case C1D: x2d=(int)((double)(getmaxx()-186)*(double)j/(double)(csys->NData()))+80;
	          break;
	case C2D: if (fabs(deltax)>1e-10) 
	            scalex=(double)(getmaxx()-186)/deltax;
                  x2d=80+(int)(scalex*(csys->Data(j,xi)-minx));
	          break;
      }
      y2d=getmaxy()-32-(int)(scaley*(csys->Data(j,yi)-miny));
      drawpoint(x2d,y2d,xWHITE,CCROSS);
    }
  }
}

double** csystem::GetData() {
  return data;
}

void csystem::Display(int t,int w,int h) {
  Window window;
  unsigned int i,j;

  type=t;
  xaxis=(int)xGetInputLineValue(wx);
  yaxis=(int)xGetInputLineValue(wy);
  zaxis=(int)xGetInputLineValue(wz);
  caxis=(int)xGetInputLineValue(wc);
  window=openwindow("Data Viewer",10,10,w,h,topshell);
  keepratio(window);
  setbkcolor(xBLACK);
  cleardevice();
  if (FindItem(window).window!=window) {
    windows[nwindows].window=window;
    windows[nwindows].item  =this;
    nwindows++;
  }
  setredrawfunction(window,cdraw);
  cdraw(window,0);
}


// the X part

csystem *csys;
int drawmode  =C1D;
int pointstyle=CDOT;
int colorstyle=0;

void draw(Widget,caddr_t,caddr_t) {
  csystem *tps;

  if (nwindows>0) {
    csys=new csystem();
    tps=topwindow.item;
    csys->SetData(tps->GetData(),tps->NData(),tps->Dim());
    csys->X((int)xGetInputLineValue(wx));
    csys->Y((int)xGetInputLineValue(wy));
    csys->Z((int)xGetInputLineValue(wz));
    csys->C((int)xGetInputLineValue(wc));
    csys->CopyStrings(tps);
  }
  csys->Display(drawmode | colorstyle | pointstyle,400+186,400+48);
}

void save(Widget,caddr_t,caddr_t) {
}

void print(Widget,caddr_t,caddr_t) {  
}

void quit(Widget,caddr_t,caddr_t) {
  exit(0);
}

void diagtype(Widget w,caddr_t,caddr_t) {
  if (!(strcmp("1D",XtName(w))))
    drawmode=C1D;
  if (!(strcmp("2D",XtName(w))))
    drawmode=C2D;
  if (!(strcmp("3D",XtName(w))))
    drawmode=C3D;
}

void pttype(Widget w,caddr_t,caddr_t) {
  if (!(strcmp("Dot",XtName(w))))
    pointstyle=CDOT;
  if (!(strcmp("Circle",XtName(w))))
    pointstyle=CCIRC;
  if (!(strcmp("Cross",XtName(w))))
    pointstyle=CCROSS;
  if (!(strcmp("Rect",XtName(w))))
    pointstyle=CQUAD;
}

void coltype(Widget w,caddr_t,caddr_t) {
  if (xGetToggleButton(w))
    colorstyle=CCOL;
  else
    colorstyle=0;
}

void about(Widget,caddr_t,caddr_t) {
  xCreateMessageBox("About","XDAT V 1.1 Beta\n\n(C) Nils Schmeisser, 1996\nFVTK FZ-Rossendorf",
		    topshell);
}

void loadLUT(char *fname) {
  FILE *f;
  char s[256];
  int i,n,c;

  if (LUT==NULL) LUT=new unsigned short[256*3];
  if ((f=fopen(fname,"rb"))!=NULL) {
    fread(LUT,256*3,2,f);
    fclose(f);
  }
  fclose(f);
  for (i=0;i<256;i++) {
    ILUT[i]=ConvRGBtoPIX(LUT[i],LUT[i+256],LUT[i+512]);
  }
}

int main(int argc,char **argv) {
  int i;
  Widget rc,rc1,rc2,rcb;
  Widget rci,rc3d,rcp;

  /* 200 * 310 */
  topshell=xInitApplication("XDAT V 1.1 Beta",&dpy,&ctx,argc,argv);
  initgraph(dpy);

  csys=new csystem();
  csys->LoadFormatted(argv[1]);
  csys->X(0);
  csys->Y(1);
  csys->C(2);

  nwindows=0;
  windows=new cwindowlist [1024];

  for (i=0;i<256;i++)
    ILUT[i]=i;
  loadLUT("standard.lut");

  rc=xCreateRowColumn(XmVERTICAL,topshell);
    rc1=xCreateRowColumn(XmHORIZONTAL,rc);
      rci=xCreateRowColumn(XmVERTICAL,xCreateFrame(rc1));
        wx=xCreateInputLine("X-axis","0",10,rci);
        wy=xCreateInputLine("Y-axis","1",10,rci);
        wz=xCreateInputLine("Z-axis","2",10,rci);
        xCreateToggleButton("Colored",False,(XtCallbackProc)coltype,rci);
        wc=xCreateInputLine(" Color","3",10,rci);
      rc2=xCreateRowColumn(XmVERTICAL,rc1);
        xCreateLabel("Diagramstyle",rc2);
        rc3d=xCreateRadioBox(xCreateFrame(rc2));
          xCreateToggleButton("1D",True,(XtCallbackProc)diagtype,rc3d);
          xCreateToggleButton("2D",False,(XtCallbackProc)diagtype,rc3d);
//          xCreateToggleButton("3D",False,(XtCallbackProc)diagtype,rc3d);
        xCreateLabel("Pointstyle",rc2);
        rcp=xCreateRadioBox(xCreateFrame(rc2));
          xCreateToggleButton("Dot",True,(XtCallbackProc)pttype,rcp);
//          xCreateToggleButton("Circle",False,(XtCallbackProc)pttype,rcp);
          xCreateToggleButton("Cross",False,(XtCallbackProc)pttype,rcp);
          xCreateToggleButton("Rect",False,(XtCallbackProc)pttype,rcp);
    rcb=xCreateRowColumn(XmHORIZONTAL,rc);
      xCreatePushButton("Draw",kbNO,(XtCallbackProc)draw,rcb);
//      xCreatePushButton("Save",kbNO,(XtCallbackProc)save,rcb);
//      xCreatePushButton("Print",kbNO,(XtCallbackProc)print,rcb);
      xCreatePushButton("About",kbNO,(XtCallbackProc)about,rcb);
      xCreatePushButton("Quit",kbNO,(XtCallbackProc)quit,rcb);

  XtAppMainLoop(ctx);
  return 0;
}

