/*
  File:        xapp.c
  Author:      Nils Schmeisser / Jan Friis Joergensen
  ----------------------------------------------------------------------
  Description: Basic and simplified functions for X programming
  ----------------------------------------------------------------------
  Notes:
  Date:        29.10.1993
  ----------------------------------------------------------------------
*/ 

#include <strstream.h>
#include <iomanip.h>

#include "xapp.h"

Cursor xArrowCursor;
Cursor xHourglassCursor;
Cursor xPencilCursor;

const int xMAXWINDOWS=200;

static Window AppWindows[xMAXWINDOWS];
static int nAppWindows=0;

/*--- Implementation ---------------------*/

static void Error(char *s) {
  cout << "XAPP error message: " << s << "\n";
}

/*--- Register Window ---------------------------------------------------------------------*/

void xRegisterWindow(Window w) {
  int i=0,j=-1;
  
  if (nAppWindows>0) {
    for (i=0;i<nAppWindows;i++) if (AppWindows[i]==w) j=i;
    if (j<0) if (nAppWindows<xMAXWINDOWS) {
      AppWindows[nAppWindows]=w;
      nAppWindows++;
    } else
      Error("To many windows, close one ore more to allow registering.");
  } else {
    AppWindows[0]=w;
    nAppWindows=1;
  }
}

void xRegisterWindow(Widget w) { xRegisterWindow(XtWindow(w)); }

/*--- Unregister Window -------------------------------------------------------------------*/

void xUnregisterWindow(Window w) {
  int i,j=-1;
 
  if (nAppWindows>0) {
    for (i=0;i<nAppWindows;i++) if (AppWindows[i]==w) j=i;
    if (j>=0) {
      if (nAppWindows>1)
	for (i=j+1;i<nAppWindows;i++) AppWindows[i-1]=AppWindows[i];
      nAppWindows--;
    }
  }
}

void xUnregisterWindow(Widget w) { xUnregisterWindow(XtWindow(w)); }

/*--- Create Popup Shell ------------------------------------------------------------------*/

Widget xCreatePopupShell(char *name,int x,int y,int w,int h,Widget Parent) {
  Widget PopupShell;
  int n;
  Arg args[10];
   
  n=0;
  XtSetArg(args[n], XmNx, x);  n++;
  XtSetArg(args[n], XmNy, y);  n++;
  XtSetArg(args[n], XmNwidth, w); n++;
  XtSetArg(args[n], XmNheight,h); n++;
  XtSetArg(args[n], XmNallowShellResize, True); n++;
  PopupShell=XtCreatePopupShell(name,applicationShellWidgetClass,Parent,args,n); 
  XtRealizeWidget(PopupShell);
  return(PopupShell);
}

/*--- Popup a PopupShell ------------------------------------------------------------------*/

void xPopup(Widget popupshell,XtGrabKind grab,int reg) 
{
  XtPopup(popupshell,grab);
  if (reg!=0) 
    xRegisterWindow(popupshell);
}

/*--- Podown a PopupShell -----------------------------------------------------------------*/

void xPopdown(Widget popupshell,int reg) {
  XtPopdown(popupshell);
  if (reg!=0) xUnregisterWindow(popupshell);
}

/*--- Create X Application Shell ----------------------------------------------------------*/

Widget xCreateAppShell(char *name,int x,int y,int w,int h,Display *display) {
  Widget Shell;
  int n;
  Arg args[6];

  n=0;
  XtSetArg(args[n], XmNx, x);  n++;
  XtSetArg(args[n], XmNy, y);  n++;
  XtSetArg(args[n], XmNwidth, w); n++;
  XtSetArg(args[n], XmNheight,h); n++;
  XtSetArg(args[n], XmNallowShellResize, True); n++;
  XtSetArg(args[n], XmNallowOverlap, True); n++;
  Shell=XtAppCreateShell(name,NULL,applicationShellWidgetClass,display,args,n);
  XtRealizeWidget(Shell);
  return(Shell);
}

/*--- Create RowColumn --------------------------------------------------------------------*/

Widget xCreateRowColumn(int flag,Widget Parent) {
  Widget Window;
  int n=0;
  Arg args[10];

  XtSetArg(args[n], XmNorientation,flag);   n++; 
  Window=XmCreateRowColumn(Parent,"RowColumn",args,n);
  XtManageChild(Window); 
  return(Window);
}

/*--- Create DrawingArea ------------------------------------------------------------------*/

Widget xCreateDrawingArea(int width,int height,Widget parent) {
  Arg args[10];
  int n=0;
  Widget DrawingArea;

  XtSetArg(args[n],XmNwidth,width); n++;
  XtSetArg(args[n],XmNheight,height); n++;
  DrawingArea=XmCreateDrawingArea(parent,"DrawingArea",args,n);
  XtManageChild(DrawingArea);
  return(DrawingArea);
}

/*--- Create RadioBox ---------------------------------------------------------------------*/

Widget xCreateRadioBox(Widget Parent) {
  Widget Window;
  int n;
  Arg args[10];

  n=0;  
  XtSetArg(args[n], XmNnumColumns, 1); n++;
  Window=XmCreateRadioBox(Parent,"RadioBox",args,n);
  XtManageChild(Window); 
  return(Window);
}

/*--- Create Frame ------------------------------------------------------------------------*/

Widget xCreateFrame(Widget Parent) {
  int n=0;
  Arg args[10];
  Widget Frame;

  XtSetArg(args[n],XmNmarginWidth,2); n++;
  XtSetArg(args[n],XmNmarginHeight,2); n++;
  XtSetArg(args[n],XmNshadowType,XmSHADOW_ETCHED_IN); n++;
  Frame=XmCreateFrame(Parent,"Frame",args,n);
  XtManageChild(Frame);
  return(Frame);
}

/*--- Create ListBox ----------------------------------------------------------------------*/

Widget xCreateList(int h,Widget Parent) {
  Widget List;
  int n=0;
  Arg args[10];

  XtSetArg(args[n], XmNheight,h); n++;
  XtSetArg(args[n],XmNselectionPolicy,XmMULTIPLE_SELECT); n++;
  List=XmCreateList(Parent,"List",args,n);
  XtManageChild(List);
  return(List);
}
  
/*--- Create Scrolled ListBox -------------------------------------------------------------*/

Widget xCreateScrolledList(int h,Widget Parent) {
  Widget List;
  int n=0;
  Arg args[10];

  XtSetArg(args[n], XmNheight,h); n++;
  XtSetArg(args[n],XmNselectionPolicy,XmMULTIPLE_SELECT); n++;
  List=XmCreateScrolledList(Parent,"List",args,n);
  XtManageChild(List);
  return(List);
}

/*--- Create Scrollbar with (min,max) and orientation -------------------------------------*/

Widget xCreateScrollBar(int min,int max,int flag,XtCallbackProc Proc,Widget Parent) {
  Arg args[10];
  int n=0;
  Widget w;

  XtSetArg(args[n],XmNminimum,min); n++;
  XtSetArg(args[n],XmNmaximum,max); n++;
  XtSetArg(args[n],XmNorientation,flag); n++;
  w=XmCreateScrollBar(Parent,"scrollbar",args,n);
  if (Proc!=NULL) XtAddCallback(w,XmNvalueChangedCallback,Proc,NULL);
  XtManageChild(w);
  return (w);
}

/*--- Initialize X Standard cursors -------------------------------------------------------*/

void xInitStandardCursor(Display *display) {
  
  xArrowCursor=XCreateFontCursor(display,XC_left_ptr);
  xHourglassCursor=XCreateFontCursor(display,XC_watch);
  xPencilCursor=XCreateFontCursor(display,XC_pencil);
}

/*--- Initialize X Application ------------------------------------------------------------*/

Widget xInitApplication(char *app_name,Display **app_display,XtAppContext *app_context,int argc,char **argv) {
  Widget TopShell;
  Cardinal cargc=argc;
  Arg args[10];
  int n=0;

  XtToolkitInitialize();
  (*app_context)=XtCreateApplicationContext();
  (*app_display)=XtOpenDisplay(*app_context,(String)NULL,(String)NULL,app_name,(XrmOptionDescRec*)NULL,0,&argc,argv);

  
  n=0;
  XtSetArg(args[n],XmNwidth,2); n++;
  XtSetArg(args[n],XmNheight,2); n++;
  
  TopShell=XtAppCreateShell((String)NULL,app_name,applicationShellWidgetClass,*app_display,args,n); 

  xInitStandardCursor(*app_display);
  XtRealizeWidget(TopShell);

  return(TopShell);
}

/*--- Create ToggleButton -----------------------------------------------------------------*/

Widget xCreateToggleButton(char *Text,Boolean value,XtCallbackProc Proc,Widget Parent) {
  int n;
  Widget Button;
  XmString xstr;
  Arg wargs[10];

  n=0;
  xstr = XmStringCreate(Text,XmSTRING_DEFAULT_CHARSET);
  XtSetArg(wargs[n], XmNlabelString, xstr); ++n;
  Button = XmCreateToggleButtonGadget(Parent,Text, wargs, n);
  XmStringFree(xstr);
  XtAddCallback(Button, XmNvalueChangedCallback,Proc, NULL);
  XtManageChild(Button);
  XmToggleButtonSetState(Button,value,TRUE);
  return(Button);
}

int xGetToggleButton(Widget w) {
  Arg args[1];
  unsigned char value;

  XtSetArg(args[0],XmNset,&value);
  XtGetValues(w,args,1);
  if (value)
    return 1;
  else
    return 0;
}

void xSetToggleButton(Widget w,int state) {
  Arg args[1];
  unsigned char value=state;

  XtSetArg(args[0],XmNset,&value);
  XtSetValues(w,args,1);
  if (state)
    XmToggleButtonSetState(w,TRUE,TRUE);
  else
    XmToggleButtonSetState(w,FALSE,TRUE);
}

/*--- Create Label ------------------------------------------------------------*/

Widget xCreateLabel(char *name,Widget Parent) {
  Widget Label;

  Label=XmCreateLabel(Parent,name,NULL,0);  
  XtManageChild(Label);
  return(Label);
}

/*--- Create InputLine -------------------------------------------------------------------*/

Widget xCreateInputLine(char *name,char *value,int length,Widget Parent) {
  Widget Input_Text,rc;
  int n=0;
  Arg      args[10];

  XtSetArg(args[n], XmNnumColumns,2); n++;  
  XtSetArg(args[n], XmNpacking, XmPACK_COLUMN);   n++; 
  XtSetArg(args[n], XmNorientation,XmVERTICAL);   n++; 
  rc=XmCreateRowColumn(Parent,"RowColumn",args,n);
  XtManageChild(rc); 

  xCreateLabel(name,rc);

  n=0;
  XtSetArg(args[n],XmNwidth,length*8); n++;
  XtSetArg(args[n],XmNheight,8); n++;
  XtSetArg(args[n], XmNwidth,  length); n++; 
  XtSetArg(args[n], XmNresizeWidth, False); n++;
  XtSetArg(args[n], XmNresizeHeight, False); n++;
  Input_Text=XmCreateText(rc,"Input_Line",args,n);
  XmTextSetString(Input_Text,value);
  XtManageChild(Input_Text);
  return(Input_Text);
}

void xSetCursor(Display *display,Window window,Cursor cursor) {
  XDefineCursor(display,window,cursor);
}

void xSetCursor(Display *display,Cursor cursor) {
  int i;
  
  for (i=0;i<nAppWindows;i++) 
    XDefineCursor(display,AppWindows[i],cursor);
}

/*--- Get value from input line -----------------------------------------------*/

float xGetInputLineValue(Widget widget) {
  char buffer[256];
  float temp;

  strcpy(buffer,XmTextGetString(widget));
  sscanf(buffer,"%f",&temp);
  return(temp);
}

char *xGetInputLineString(Widget widget) {
  char *buffer=new char [strlen(XmTextGetString(widget))+1];

  strcpy(buffer,XmTextGetString(widget));
  return buffer;
}

/*--- Set value to input line -------------------------------------------------*/

void xSetInputLineValue(Widget W,long no) {
  char sval[256];

  sprintf(sval,"%d",no);  
  XmTextSetString(W,sval);
}

void xSetInputLineValue(Widget W,int no) {
  char sval[256];

  sprintf(sval,"%d",no);  
  XmTextSetString(W,sval);
}


void xSetInputLineValue(Widget W,float no) {
  char sval[256];

  sprintf(sval,"%f",no);  
  XmTextSetString(W,sval);
}

void xSetInputLineValue(Widget W,double no) {
  char sval[256];

  sprintf(sval,"%lf",no);  
  XmTextSetString(W,sval);
}

void xSetInputLineValue(Widget widget,char *value) {
  XmTextSetString(widget,value);
}

/*--- Create Push Button ------------------------------------------------------*/

Widget xCreatePushButton(char *label,char mnemonic,XtCallbackProc Proc,Widget Parent) {
  int n;
  Arg args[10];
  Widget Button;

  n=0;
  XtSetArg(args[n], XmNx, 220);  n++;
//  SetArg(args, XmNx, 220,n);
  XtSetArg(args[n], XmNy, 10);  n++;
  XtSetArg(args[n], XmNwidth, 100); n++;
  XtSetArg(args[n], XmNheight,30); n++;
  if (mnemonic!=kbNO) 
    {XtSetArg(args[n], XmNmnemonic,mnemonic); 
     n++;}
  Button=XmCreatePushButton(Parent,label,args,n);
  XtManageChild(Button);
  if (Proc!=NULL) XtAddCallback(Button,XmNactivateCallback,Proc,NULL);
  return(Button);
}  

/*--- Create Arrow Button ------------------------------------------------------*/

Widget xCreateArrowButton(int direction,XtCallbackProc Proc,Widget Parent) {
  int n;
  Arg args[10];
  Widget Button;

  n=0;
  XtSetArg(args[n], XmNarrowDirection,direction);  n++;
  Button=XmCreateArrowButton(Parent,"ArrowButton",args,n);
  XtManageChild(Button);
  XtAddCallback(Button,XmNactivateCallback,Proc,NULL);
  return(Button);
}

/*--- Adding Callback function -------------------------------------------------*/

void xAddCallback(Widget widget,XtCallbackProc Proc) {
  XtAddCallback(widget,XmNactivateCallback,Proc,NULL);
}

void xSetLabel(Widget w, char *label) {
  Arg args[6];

  XmString str=XmStringCreate(label,XmSTRING_DEFAULT_CHARSET);
  XtSetArg(args[0],XmNlabelString,str);
  XtSetValues(w,args,1);
}

char *xGetLabel(Widget widget) {
  Arg args[6];
  char *ret;
  XmString str;

  XtSetArg(args[0],XmNlabelString,&str);
  XtGetValues(widget,args,1);
//  XmStringGetLtoR(str,XmFONTLIST_DEFAULT_TAG,&ret);
  ret=(char*)NULL;
  return ret;
}

/*--- GetCursorPosition in respect to window -------------------------*/

Boolean xGetMousePosition(Display *display,Window window,int *x,int *y) {
  Window root,child;
  int X,Y;
  unsigned int button;

  return XQueryPointer(display,window,&root,&child,&X,&Y,x,y,&button);
}

unsigned int xGetMouseButton(Display *display) {
  Window root,child,window;
  int x,y,X,Y;
  unsigned int button;

  window=XDefaultRootWindow(display);
  if (XQueryPointer(display,window,&root,&child,&X,&Y,&x,&y,&button)) return(button); 
    else return(0);
}

Window xWhereisMouse(Display *display,int *x,int *y) {
  Window root,child,window;
  int X,Y;
  unsigned int button;

  window=XDefaultRootWindow(display);
  if (XQueryPointer(display,window,&root,&child,&X,&Y,x,y,&button)) return child; else return 0;
}

//--- MessageBox -----------------------------------------------------

void CloseMessageBox(Widget widget,caddr_t client_data,caddr_t out_data) {
  Widget w=XtParent(XtParent(widget));
  xPopdown(w);
  XtDestroyWidget(w);
}

Widget xCreateMessageBox(char* title,char* text,Widget parent) {
  Widget rc;
  Widget m;
  char s[256];
  int i,n;
  Arg args[10];
  Widget Button;

  m=xCreatePopupShell(title,50,50,strlen(text)*8+32,200,parent);
    rc=xCreateRowColumn(XmVERTICAL,m);
      n=0;
      for (i=0;i<strlen(text);i++) {
	if (text[i]!='\n') {
	  s[n]=text[i];
	  n++;
	} else {
	  s[n]=0;
	  xCreateLabel(s,rc);
	  n=0;
	}
      }
      xCreatePushButton(" OK ",kbNO,(XtCallbackProc)CloseMessageBox,rc);

  xPopup(m);
  return m;
}















