/*--------------------------------------------------------------------
  File:        xapp.h
  Author:      Nils Schmeisser
  ----------------------------------------------------------------------
  Description: Basic Functions for X programming
  ----------------------------------------------------------------------
  Notes:       link application to "xapp.o"
  Date:        29.10.1993
  ----------------------------------------------------------------------
*/ 

#ifndef XAPP
#define XAPP
#define XTFUNCPROTO

#ifdef __cplusplus
extern "C"{
#define signed
#endif

/*--- Common includes --------------------*/

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/cursorfont.h>
#include <Xm/MenuShell.h>

/*--- Widget includes -------------------*/

#include <Xm/Xm.h>
#include <Xm/RowColumn.h>
#include <Xm/DrawingA.h>
#include <Xm/MainW.h>
#include <Xm/MessageB.h>

#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>

#include <Xm/ArrowB.h>
#include <Xm/PushB.h>
#include <Xm/CascadeB.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>
#include <Xm/ScrollBar.h>

#include <Xm/List.h>

#include <stdio.h>

#ifdef __cplusplus
}
#endif

/*--- constants ------------------------*/

const int kbEsc		=9;
const int kbF1		=67;
const int kbF2		=68;
const int kbF3		=69;
const int kbF4		=70;
const int kbF5		=71;
const int kbF6		=72;
const int kbF7		=73;
const int kbF8		=74;
const int kbF9		=75;
const int kbF10		=76;
const int kbF11		=95;
const int kbF12		=96;
const int kbPrintScreen	=111;
const int kbScrollLock	=78;
const int kbPause	=110;
const int kbApostroph	=49;
const int kb1		=10;
const int kb2		=11;
const int kb3		=12;
const int kb4		=13;
const int kb5		=14;
const int kb6		=15;
const int kb7		=16;
const int kb8		=17;
const int kb9		=18;
const int kb0		=19;
const int kbMinus	=20;
const int kbEqual	=21;
const int kbBackSlash	=51;
const int kbBackspace	=22;
const int kbTAB		=23;
const int kbq		=24;
const int kbw		=25;
const int kbe		=26;
const int kbr		=27;
const int kbt		=28;
const int kby		=29;
const int kbu		=30;
const int kbi		=31;
const int kbo		=32;
const int kbp		=33;
const int kbOpenBracket	=34;
const int kbCloseBracket=35;
const int kbCapsLock	=66;
const int kba		=38;
const int kbs		=39;
const int kbd		=40;
const int kbf		=41;
const int kbg		=42;
const int kbh		=43;
const int kbj		=44;
const int kbk		=45;
const int kbl		=46;
const int kbSemicolon	=47;
const int kbHighKomma	=48;
const int kbEnter	=36;
const int kbLShift	=50;
const int kbz		=52;
const int kbx		=53;
const int kbc		=54;
const int kbv		=55;
const int kbb		=56;
const int kbn		=57;
const int kbm		=58;
const int kbKomma	=59;
const int kbDot		=60;
const int kbSlash	=61;
const int kbRShift	=62;
const int kbLCtrl	=37;
const int kbLAlt	=64;
const int kbSpace	=65;
const int kbRAlt	=113;
const int kbRCtrl	=109;
const int kbIns		=106;
const int kbHome	=97;
const int kbPgUp	=99;
const int kbDel		=107;
const int kbEnd		=103;
const int kbPgDown	=105;
const int kbUp		=98;
const int kbLeft	=100;
const int kbRigth	=102;
const int kbDown	=104;
const int kbNumLock     =77;
const int kbNumSlash	=112;
const int kbNumStar	=63;
const int kbNumMinus	=82;
const int kbNum7	=136;
const int kbNum8	=137;
const int kbNum9	=138;
const int kbNum4	=139;
const int kbNum5	=140;
const int kbNum6	=141;
const int kbNumPlus	=86;
const int kbNum1	=142;
const int kbNum2	=143;
const int kbNum3	=144;
const int kbNum0	=145;
const int kbNumDot	=146;
const int kbNumEnter	=108;

const char kbNO        = '\0';
const char kbAltA      = 'a';
const char kbAltShiftA = 'A';
const char kbAltB      = 'b';
const char kbAltShiftB = 'B';
const char kbAltC      = 'c';
const char kbAltShiftC = 'C';
const char kbAltD      = 'd';
const char kbAltShiftD = 'D';
const char kbAltE      = 'e';
const char kbAltShiftE = 'E';
const char kbAltF      = 'f';
const char kbAltShiftF = 'F';
const char kbAltG      = 'g';
const char kbAltShiftG = 'G';
const char kbAltH      = 'h';
const char kbAltShiftH = 'H';
const char kbAltI      = 'i';
const char kbAltShiftI = 'I';
const char kbAltJ      = 'j';
const char kbAltShiftJ = 'J';
const char kbAltK      = 'k';
const char kbAltShiftK = 'K';
const char kbAltL      = 'l';
const char kbAltShiftL = 'L';
const char kbAltM      = 'm';
const char kbAltShiftM = 'M';
const char kbAltN      = 'n';
const char kbAltShiftN = 'N';
const char kbAltO      = 'o';
const char kbAltShiftO = 'O';
const char kbAltP      = 'p';
const char kbAltShiftP = 'P';
const char kbAltQ      = 'q';
const char kbAltShiftQ = 'Q';
const char kbAltR      = 'r';
const char kbAltShiftR = 'R';
const char kbAltS      = 's';
const char kbAltShiftS = 'S';
const char kbAltT      = 't';
const char kbAltShiftT = 'T';
const char kbAltU      = 'u';
const char kbAltShiftU = 'U';
const char kbAltV      = 'v';
const char kbAltShiftV = 'V';
const char kbAltW      = 'w';
const char kbAltShiftW = 'W';
const char kbAltX      = 'x';
const char kbAltShiftX = 'X';
const char kbAltY      = 'y';
const char kbAltShiftY = 'Y';
const char kbAltZ      = 'z';
const char kbAltShiftZ = 'Z';

const int xLEFTBUTTON  =  256;
const int xMIDDLEBUTTON=  512;
const int xRIGHTBUTTON = 1024;

extern Cursor xArrowCursor;
extern Cursor xHourglassCursor;
extern Cursor xPencilCursor;

/*--- Interface ------------------------*/

extern Widget xInitApplication(char *app_name,Display **app_display,XtAppContext *app_context,int argc,char **argv);

extern Widget xCreateAppShell(char *name,int x,int y,int w,int h,Display *display);
extern Widget xCreatePopupShell(char *name,int x,int y,int w,int h,Widget Parent);

extern Widget xCreateDrawingArea(int width,int height,Widget parent);
extern Widget xCreateRowColumn(int flag,Widget Parent);
extern Widget xCreateRadioBox(Widget Parent);
extern Widget xCreateFrame(Widget Shell);

extern Widget xCreateList(int h,Widget Parent);
extern Widget xCreateScrolledList(int h,Widget Parent);

extern Widget xCreateToggleButton(char *Text,Boolean value,XtCallbackProc Proc,Widget Parent);
extern int xGetToggleButton(Widget w);
extern void xSetToggleButton(Widget w,int state);
extern Widget xCreatePushButton(char *label,char mnemonic,XtCallbackProc Proc,Widget Parent);
extern Widget xCreateArrowButton(int direction,XtCallbackProc Proc,Widget Parent);

extern Widget xCreateScrollBar(int min,int max,int flag,XtCallbackProc Proc,Widget Parent);

extern Widget xCreateLabel(char *name,Widget Parent);
extern Widget xCreateInputLine(char *name,char *value,int length,Widget Parent);
extern void xSetLabel(Widget widget,char* label);
extern char *xGetLabel(Widget w);

extern void xSetInputLineValue(Widget widget,float value);
extern void xSetInputLineValue(Widget widget,double value);
extern void xSetInputLineValue(Widget widget,long value);
extern void xSetInputLineValue(Widget widget,int value);
extern void xSetInputLineValue(Widget widget,char *value);
extern float xGetInputLineValue(Widget widget);
extern char *xGetInputLineString(Widget widget);

extern void xAddCallback(Widget widget,XtCallbackProc Proc);
extern void xPopup(Widget popupshell,XtGrabKind grab=XtGrabNonexclusive,int reg=TRUE);
extern void xPopdown(Widget popupshell,int reg=TRUE);
extern void xSetCursor(Display *display,Window window,Cursor cursor);
extern void xSetCursor(Display *display,Cursor cursor);

extern unsigned int xGetMouseButton(Display *display);
extern Window xWhereisMouse(Display *display,int *x,int *y);
extern Boolean xGetMousePosition(Display *display,Window window,int *x,int *y);

extern void xRegisterWindow(Window w);
extern void xRegisterWindow(Widget w);
extern void xUnregisterWindow(Window w);
extern void xUnregisterWindow(Widget w);

extern Widget xCreateMessageBox(char* title,char* text,Widget parent);

#endif














