/*--------------------------------------------------------------------
  File:        xgraphics.h
  Author:      Nils Schmeisser
  ----------------------------------------------------------------------
  Description: Graphics module for X applications
  ----------------------------------------------------------------------
  Notes:       
  Date:        3.12.1993
  ----------------------------------------------------------------------
*/ 

#ifndef XGRAPH
  #define XGRAPH

  #define XTFUNCPROTO

  #include <sys/param.h>
  #include <Xm/Xm.h>

  #ifndef MAXINT
    #define MAXINT -65535
  #endif

  typedef void (*XGraphicsRedrawFunction)(Window,int);
  typedef XPoint polypoint;

  #define RedrawRequest ResizeRequest

  extern int xBLACK;
  extern int xBLUE;
  extern int xGREEN;
  extern int xCYAN;
  extern int xRED;
  extern int xMAGENTA;
  extern int xBROWN;
  extern int xLIGHTGRAY;
  extern int xDARKGRAY;
  extern int xLIGHTBLUE;
  extern int xLIGHTGREEN;
  extern int xLIGHTCYAN;
  extern int xLIGHTRED;
  extern int xLIGHTMAGENTA;
  extern int xYELLOW;
  extern int xWHITE;

  extern void initgraph(Display* =NULL);
  extern void closegraph(Display* =NULL);
  extern Display *getgraphicsdisplay();
  extern void setgraphicsdisplay(Display *dpy);

  extern Window openwindow(char *name,int x,int y,int sizex,int sizey,Widget parent,Widget *da=NULL);
  extern Window openwindow(char *name,int x,int y,int sizex,int sizey,Widget parent,
			   XGraphicsRedrawFunction f,Widget *da=NULL);
extern void closewindow(Window window);
extern void setwindow(Window window);
extern void setredrawfunction(Window window,XGraphicsRedrawFunction f);

//--- full calling style --------------------------------------------------------------------

extern void cleardevice(Window =0);

extern int getmaxcolor();
extern void setcolor(Window window,int color);
extern int getfgcolor(Window window=0);
extern void setbkcolor(Window window,int color);
extern int getbkcolor(Window window=0);
extern int ConvRGBtoPIX(int red,int green,int blue);
extern void setrgbpalette(int colornum, int red, int green, int blue);
extern void getrgbpalette(int colornum, int &red, int &green, int &blue);
void keepratio(Window window,Boolean keepit);

extern int getmaxx(Window =0);
extern int getmaxy(Window =0);
extern void getsize(Window window,int &w,int &h);
extern void getposition(Window window,int &x,int &y);

extern void moveto(Window window,int x,int y);

extern int getpixel(Window window,int x,int y);
extern void putpixel(Window window,int x,int y,int =-MAXINT);
extern void putrgbpixel(Window window,int x,int y,int r,int g,int b);

extern void line(Window window,int x1,int y1,int x2,int y2,int =-MAXINT);
extern void lineto(Window window,int x,int y,int =-MAXINT);
extern void drawpoly(Window window,int num,polypoint *points,int =-MAXINT);
extern void fillpoly(Window window,int num,polypoint *points,int =-MAXINT);

extern void rectangle(Window window,int xa,int ya,int xb,int yb,int = -MAXINT);
extern void bar(Window window,int xa,int ya,int xb,int yb,int = -MAXINT);

extern void arc(Window window,int x,int y,int startangle,int stopangle,int r,int =-MAXINT);
extern void circle(Window window,int x,int y,int r,int =-MAXINT);
extern void fillarc(Window window,int x,int y,int startangle,int stopangle,int r,int =-MAXINT);
extern void fillcircle(Window window,int x,int y,int r,int =-MAXINT);

extern unsigned long imagesize(Window window,int x1,int y1,int x2,int y2);
extern void getimage(Window window,int x,int y,unsigned short w,unsigned short h,void *bitmap);
extern void putimage(Window window,int x,int y,void *bitmap,int flag);
extern void getimage(int x,int y,int w,int h,void *bitmap);
extern void putimage(int x,int y,void *bitmap,int flag);

extern void setfont(char *fcode);
extern char *getfont();
extern int getfontwidth();
extern int getfontheight();
extern void outtextxy(Window window,int x,int y,char *s,int =-MAXINT);
extern void outtext(Window window,char *s,int =-MAXINT);

//--- without window calling style --------------------------------------------------

extern void setcolor(int color);
extern void setbkcolor(int color);
void keepratio(Boolean keepit);

extern void getsize(int &w,int &h);
extern void getposition(int &x,int &y);

extern unsigned int readkey();
extern Boolean getpointer(Window window,int *x,int *y);
extern Boolean getpointer(int *x,int *y);
extern unsigned int getbutton(Window window);
extern unsigned int getbutton();

extern void moveto(int x,int y);

extern int getpixel(int x,int y);
extern void putpixel(int x,int y,int =-MAXINT);
extern void putrgbpixel(int x,int y,int r,int g,int b);

extern void line(int x1,int y1,int x2,int y2,int =-MAXINT);
extern void lineto(int x,int y,int =-MAXINT);
extern void drawpoly(int num,polypoint *points,int =-MAXINT);
extern void fillpoly(int num,polypoint *points,int =-MAXINT);

extern void rectangle(int xa,int ya,int xb,int yb,int = -MAXINT);
extern void bar(int xa,int ya,int xb,int yb,int = -MAXINT);

extern void arc(int x,int y,int startangle,int stopangle,int r,int =-MAXINT);
extern void circle(int x,int y,int r,int =-MAXINT);
extern void fillarc(int x,int y,int startangle,int stopangle,int r,int =-MAXINT);
extern void fillcircle(int x,int y,int r,int =-MAXINT);

extern void outtextxy(int x,int y,char *s,int =-MAXINT);
extern void outtext(char *s,int =-MAXINT);

#endif

